/* Library to make operations available in C */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <unistd.h>
#include <time.h>

#include "Alta.h"
#include "Ascent.h"
#include "AltaF.h"
#include "Aspen.h"
#include "Quad.h"
#include "FindDeviceUsb.h"
#include "FindDeviceEthernet.h"
#include "CameraInfo.h"
#include "apgSampleCmn.h"

#include "libapogeedrv.h"


/* Local storage */

int exp_width = 0;
int exp_height = 0;

/* Master object */

ApogeeCam *apogeeCamera;


/* Function prototype */

static void apogee_check();


int apogee_open(unsigned int id)
{
  std::string modelStr;
  std::string ioInterface("usb");
  CamModel::PlatformType model;
  FindDeviceUsb lookUsb;
  std::string msg = lookUsb.Find();                
  std::string addr = apgSampleCmn::GetUsbAddress( msg );
  id = apgSampleCmn::GetID( msg );
  unsigned short frmwrRev = apgSampleCmn::GetFrmwrRev( msg );
  modelStr = apgSampleCmn::GetItemFromFindStr(  msg, "model=" );
  model = CamModel::GetPlatformType(modelStr);

  switch (model)
     {
        case CamModel::ALTAU:
        case CamModel::ALTAE:
         apogeeCamera = new Alta();
         break;

        case CamModel::ASPEN:
         apogeeCamera = new Aspen();
         break;

        case CamModel::ALTAF:
         apogeeCamera = new AltaF();
         break;

        case CamModel::ASCENT:
         apogeeCamera = new Ascent();
         break;

        case CamModel::QUAD:
         apogeeCamera = new Quad();
         break;

        default:
         fprintf(stderr,  "Model %s is not supported by the Apogee driver.", modelStr.c_str());
         return false;
         break;
  }
  apogeeCamera->OpenConnection(ioInterface, addr, frmwrRev, id);
  apogeeCamera->Init();
  if (!apogeeCamera->IsInitialized())
  {
    return (-1);
  }
  
  apogeeCamera->SetLedAState (Apg::LedState_Expose);
  apogeeCamera->SetLedBState (Apg::LedState_AtTemp);

  return (0);
}

/* Return the camera operating parameters */

void apogee_get_values(int *roiw, int *roih, int *osw, int *osh, 
  int *binw, int *binh, double *mintemp)
{
  apogee_check();

  *roiw = apogeeCamera->GetRoiNumCols();
  *roih = (int)apogeeCamera->GetRoiNumRows();
  *osw =  (int)apogeeCamera->GetNumOverscanCols();
  *osh =  0;
  *binw = apogeeCamera->GetRoiBinCol();
  *binh = apogeeCamera->GetRoiBinRow();
  *mintemp = -30.0; /*TODO*/
}


/* Start an exposure without error checking */

int apogee_expose(double exposure, int x, int y, int height, int width, 
  int shutter_flag)
{
  int status;
  
  exp_width = width;
  exp_height = height;
  apogeeCamera->SetRoiStartCol(x);
  apogeeCamera->SetRoiStartRow(y);
  apogeeCamera->SetRoiNumCols(width);
  apogeeCamera->SetRoiNumRows(height);
  
  /* No overscan */
  apogeeCamera->SetDigitizeOverscan(0);
  
  /* Single image per download */
  apogeeCamera->SetImageCount(1);

  /* Start an exposure */
  apogeeCamera->StartExposure(exposure,shutter_flag);
      
  return(1);
}


/* Stop an exposure without reading out */

int apogee_stop_exposure(void)
{
  apogee_check();
  apogeeCamera->StopExposure(0);
  return(1);
}

/* Check camera status */
/* Return 2 if the current exposure is complete */
/* Return 1 if exposure is integrating */
/* Return 0 otherwise */

int apogee_get_status(void)
{
  apogee_check();
  if  (apogeeCamera->GetImagingStatus() == Apg::Status_ImageReady)
  {
    return(2);
  }
  return(0);  
}

/* Return the image based on data size from exposure request */

int apogee_get_image(unsigned short int *image)
{  

  long status;
  unsigned long count;
  unsigned short width;
  unsigned short height;
  std::vector<unsigned short> pImageData;
  
  width = exp_width;
  height = exp_height;
  count = width*height;
  apogeeCamera->GetImage (pImageData);
  copy(pImageData.begin(), pImageData.end(), image);

  return(1);
}


/* Set cooler target temperature */

void apogee_set_cooler_temperature(double newtemp)
{    
  apogee_check();
  apogeeCamera->SetCoolerSetPoint(newtemp);
}


/* Set cooler state */

void apogee_set_cooler_state(int state)
{
  apogee_check();
  
  if (state == 0) 
  {
    apogeeCamera->SetCooler( (bool) 0 );
  } 
  else 
  {
    apogeeCamera->SetCooler( (bool) 1 );
  }
}


/* Report urgent cooler tempperature status */
/*   0 = off                                */
/*   1 = ramping to set point               */              
/*   2 = at set point                       */                     

int apogee_get_cooler_temperature(double *apogeetemp)
{
  int status;

  apogee_check();

  *apogeetemp = apogeeCamera->GetTempCcd();
  status = apogeeCamera->GetCoolerStatus();
  if (status > 2)
  {
    status = 2;
  }
  return (status);
}

/* Set fan speed from  0 to 3 */

void apogee_set_fan(int speed)
{
  apogeeCamera->SetFanMode((Apg::FanMode)speed,0);   
}

/* Actuate a guider relay */

int apogee_guide(int relay, double ontime)
{
  return(0);
}

/* Actuate the adaptive optics tip-tilt */

int apogee_tiptilt(int ao_x, int ao_y)
{
  return(0);
}


/* Return pointers to sensor and camera name */
/* Caller should neither modify nor free these strings. */

void apogee_get_info(char **sensor, char **camera)
{
  const char *lsensor, *lcamera;
  lsensor = apogeeCamera->GetSensor().c_str();
  lcamera = apogeeCamera->GetModel().c_str();
//  strcpy(*sensor,lsensor);
//  strcpy(*camera,lcamera);
}



/* Check that the alta object has been created */ 

static void apogee_check()
{
  if (!apogeeCamera->IsInitialized())
  {
    fprintf (stderr, "Bug! Alta used before open\n");
    exit(1);
  }
}


